package com.gic.auth.service.outer.impl;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.constant.DeleteFlagConstants;
import com.gic.auth.dto.MenuRoleDTO;
import com.gic.auth.dto.RoleDTO;
import com.gic.auth.dto.RoleListDTO;
import com.gic.auth.entity.TabSysRole;
import com.gic.auth.entity.TabSysUserRole;
import com.gic.auth.qo.RoleListQO;
import com.gic.auth.service.MenuRoleService;
import com.gic.auth.service.RoleApiService;
import com.gic.auth.service.RoleService;
import com.gic.auth.service.UserRoleService;
import com.gic.auth.utils.ValidSplitUtils;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.utils.valid.ValidParamsUtils;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
/**
 * 
 * @ClassName: RoleApiServiceImpl
 * @Description:  
 * @author zhiwj
 * @date 2020-09-10 9:38
 */
@Service("roleApiService")
public class RoleApiServiceImpl implements RoleApiService{
    @Autowired
    private RoleService roleService;
    @Autowired
    private MenuRoleService menuRoleService;
    @Autowired
    private UserRoleService userRoleService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public ServiceResponse<Integer> save(RoleDTO dto) {
        ServiceResponse paramValid = ValidParamsUtils.allCheckValidate(dto, RoleDTO.SaveRoleValid.class);
        if (!paramValid.isSuccess()) {
            return paramValid;
        }
        dto.setUpdateTime(new Date());
        dto.setCreateTime(new Date());
        dto.setDeleteFlag(DeleteFlagConstants.NORMAL_STATUS);

        //验证名称不能重复
        if (roleService.isRepeatRoleName(dto.getEnterpriseId(), null, dto.getRoleName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "权限名称重复");
        }

        Integer roleId = roleService.save(dto);

        //删除之前的角色菜单数据
        menuRoleService.deleteByRoleId(dto.getRoleId());
        ServiceResponse<Integer> menuRoleResponse = saveMenuRole(roleId, dto.getEnterpriseId(), dto.getMenuIds());
        if (!menuRoleResponse.isSuccess()) {
            //事物回滚
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
        }
        return menuRoleResponse;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public ServiceResponse<Integer> update(RoleDTO dto) {
        TabSysRole record = roleService.getById(dto.getRoleId());
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "角色数据有误，查无数据");
        }
        ServiceResponse paramValid = ValidParamsUtils.allCheckValidate(dto, RoleDTO.EditRoleValid.class);
        if (!paramValid.isSuccess()) {
            return paramValid;
        }

        //验证名称不能重复
        if (roleService.isRepeatRoleName(record.getEnterpriseId(), dto.getRoleId(), dto.getRoleName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "权限名称重复");
        }

        roleService.update(dto);
        //删除之前的角色菜单数据
        menuRoleService.deleteByRoleId(dto.getRoleId());

        ServiceResponse<Integer> menuRoleResponse = saveMenuRole(dto.getRoleId(), record.getEnterpriseId(), dto.getMenuIds());
        if (!menuRoleResponse.isSuccess()) {
            //事物回滚
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
        }
        return menuRoleResponse;
    }

    @Override
    public ServiceResponse<RoleDTO> getById(Integer id) {
        TabSysRole record = roleService.getById(id);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "角色数据有误，查无数据");
        }
        List<MenuRoleDTO> menuRoleDTOList = menuRoleService.listMenuRoleByRoleId(id);
        StringBuilder menuId = new StringBuilder();
        for (MenuRoleDTO menuRole : menuRoleDTOList) {
            menuId.append(menuRole.getMenuId()).append(",");
        }
        RoleDTO result = EntityUtil.changeEntityNew(RoleDTO.class, record);
        String menuIds = menuId.toString();
        result.setMenuIds(menuIds.substring(0, menuIds.length() - 1));
        return ServiceResponse.success(result);
    }

    @Override
    public ServiceResponse<String> delete(Integer id) {
        TabSysRole record = roleService.getById(id);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "角色数据有误，查无数据");
        }
        //判断是否正在被用户使用
        List<TabSysUserRole> userRoleList = userRoleService.listUserRoleByRoleId(id);
        if (userRoleList.size() > 0) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "无法删除，正在被使用");
        }
        roleService.deleteById(id);
        return ServiceResponse.success(record.getRoleName());
    }

    @Override
    public ServiceResponse<Page<RoleListDTO>> pageRole(RoleListQO params) {
        com.github.pagehelper.Page page = roleService.pageRole(params);
        Page<RoleListDTO> resultPage = PageHelperUtils.changePageHelperToCurrentPage(page);
        return ServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse<List<RoleDTO>> listRole(Integer enterpriseId) {
        List<TabSysRole> list = roleService.listRole(enterpriseId);
        if (CollectionUtils.isNotEmpty(list)) {
            return ServiceResponse.success(EntityUtil.changeEntityListNew(RoleDTO.class, list));
        }
        return ServiceResponse.success(new ArrayList<>());
    }

    private ServiceResponse<Integer> saveMenuRole(Integer roleId, Integer enterpriseId, String menuIds) {
        String[] menuIdArr;
        ServiceResponse<String[]> menuIdResponse = ValidSplitUtils.validStr(menuIds);
        if (menuIdResponse.isSuccess()) {
            menuIdArr = menuIdResponse.getResult();
        } else {
            return ServiceResponse.failure(menuIdResponse.getCode(), menuIdResponse.getMessage());
        }
        List<MenuRoleDTO> list = new ArrayList<>(menuIdArr.length);
        Date now = new Date();
        for (String menuId : menuIdArr) {
            MenuRoleDTO menuRoleDTO = new MenuRoleDTO();
            menuRoleDTO.setEnterpriseId(enterpriseId);
            menuRoleDTO.setRoleId(roleId);
            menuRoleDTO.setMenuId(Integer.parseInt(menuId));
            menuRoleDTO.setCreateTime(now);
            menuRoleDTO.setUpdateTime(now);
            menuRoleDTO.setDeleteFlag(DeleteFlagConstants.NORMAL_STATUS);
            list.add(menuRoleDTO);
        }
        menuRoleService.insertForeach(list);
        return ServiceResponse.success(roleId);
    }
}
