package com.gic.auth.service.outer.impl;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

import com.gic.auth.dto.wechat.WechatUserDTO;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.constant.*;
import com.gic.auth.dto.*;
import com.gic.auth.entity.*;
import com.gic.auth.qo.UserListQO;
import com.gic.auth.service.*;
import com.gic.auth.utils.ValidSplitUtils;
import com.gic.auth.utils.qrcode.QrcodeUtils;
import com.gic.auth.utils.transfer.ManagerDataUtils;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.GICMQClientUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.constant.QrcodeTypeEnum;
import com.gic.enterprise.constants.Constants;
import com.gic.enterprise.error.ErrorCode;
import com.gic.mq.sdk.GicMQClient;
import com.gic.redis.data.util.RedisUtil;
import com.gic.store.utils.valid.ValidUtil;

/**
 * @author guojx
 * @date 2019/7/16 6:55 PM
 */
@Service("userApiService")
public class UserApiServiceImpl implements UserApiService {
    private static final Logger LOGGER = LogManager.getLogger(UserApiServiceImpl.class);
    @Autowired
    private UserService userService;
    @Autowired
    private UserRoleApiService userRoleApiService;
    @Autowired
    private UserResourceApiService userResourceApiService;
    @Autowired
    private UserRoleService userRoleService;
    @Autowired
    private UserResourceService userResourceService;
    @Autowired
    private ResourceGroupService resourceGroupService;
    @Autowired
    private RoleService roleService;
    @Autowired
    private AccountGroupRelService accountGroupRelService;
    @Autowired
    private AccountGroupService accountGroupService;
    @Autowired
    private AuditedGroupUserRelService auditedGroupUserRelService;
    @Autowired
    private UserMoveApiService userMoveApiService;
    @Autowired
    private AuditorApiService auditorApiService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public ServiceResponse<Integer> saveUser(UserDTO userDTO) {
        //valid param
        ServiceResponse paramResult = ValidUtil.allCheckValidate(userDTO, UserDTO.SaveUserValid.class);
        if (!paramResult.isSuccess()) {
            return paramResult;
        }

        //如果是普通管理员，则需要验证角色权限和资源权限
        if (userDTO.getSuperAdmin().intValue() == 0) {
            if (StringUtils.isBlank(userDTO.getUserRoleIds())) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "角色权限不能为空");
            }
            if (StringUtils.isBlank(userDTO.getUserResourceIds())) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "资源权限不能为空");
            }
        }

        if (userDTO.getSuperAdmin() == null) {
            //默认是普通管理员
            userDTO.setSuperAdmin(0);
        }
        //超级管理员不能创建多条，需要判断控制
        if (userDTO.getSuperAdmin().intValue() == 1) {
            TabSysUser adminUser = userService.getUserByEnterpriseId(userDTO.getEnterpriseId());
            if (adminUser != null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "已存在超级管理员，重复创建");
            }
        }

        if (userDTO.getPasswordType() == null) {
            //默认自定义创建密码类型
            userDTO.setPasswordType(2);
        }

        ServiceResponse validParamRes = validParamOfSaveOrUpdate(userDTO);
        if (!validParamRes.isSuccess()) {
            return ServiceResponse.failure(validParamRes.getCode(), validParamRes.getMessage());
        }

        Integer userId = userService.saveUser(userDTO);

        userRoleService.deleteByUserId(userId);
        userResourceService.deleteByUserId(userId);
        //如果账号分组有数据，则需要维护分组数据
        if (StringUtils.isNotBlank(userDTO.getAccountGroupIds())) {
            accountGroupRelService.deleteByUserId(userId, AccountGroupMemberTypeEnum.ADMIN.getCode());
        }

        ServiceResponse<Integer> response = saveRole(userId, userDTO.getEnterpriseId(), userDTO.getUserRoleIds(),
                userDTO.getUserResourceIds(), userDTO.getAccountGroupIds());
        if (response.isSuccess()) {
            return response;
        } else {
            //事物回滚
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return response;
        }
    }

    @Override
    public ServiceResponse<Integer> saveOperationUser(UserDTO userDTO) {
        if (StringUtils.isBlank(userDTO.getPhoneNumber())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "手机号不能为空");
        }
        if (StringUtils.isBlank(userDTO.getUserName())) {
            ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "用户名称不能为空");
        }
        if (userDTO.getEnterpriseId() == null) {
            ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID不能为空");
        }
        if (userService.isPhoneRepeat(null, userDTO.getPhoneNumber(), userDTO.getEnterpriseId(),
                LoginUserTypeEnum.OPERATION_USER.getCode())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "手机号码不能重复");
        }
        return ServiceResponse.success(userService.saveOperationUser(userDTO));
    }

    @Override
    public ServiceResponse<Void> updateOperationUser(Integer userId, String userName) {
        UserDTO userDTO = new UserDTO();
        userDTO.setUserId(userId);
        userDTO.setUserName(userName);
        userService.editUser(userDTO);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> updateUserPosition(Integer userId, Integer userPosition) {
        UserDTO userDTO = new UserDTO();
        userDTO.setUserId(userId);
        userDTO.setUserPosition(userPosition);
        userService.editUser(userDTO);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> updateUserOpenId(Integer userId, String openId) {
        UserDTO userDTO = new UserDTO();
        userDTO.setUserId(userId);
        userDTO.setOpenId(openId);
        userService.editUser(userDTO);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<String> getUserQrcode(Integer userId, Integer enterpriseId) {
        return ServiceResponse.success(QrcodeUtils.getTempQrCodeUrl(QrcodeTypeEnum.USER.getCode(), userId, enterpriseId));
    }

    @Override
    public ServiceResponse<WechatUserDTO> getMemberInfoByOpenId(String openId) {
        WechatUserDTO wechatUserDTO = QrcodeUtils.getMemberInfoByOpenId(openId);
        if (wechatUserDTO != null) {
            return ServiceResponse.success(wechatUserDTO);
        }
        return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "查询不到用户信息");
    }

    @Override
    public ServiceResponse<Integer> saveAdmin(UserDTO userDTO) {
        //valid param
        ServiceResponse paramResult = ValidUtil.allCheckValidate(userDTO, UserDTO.SaveUserValid.class);
        if (!paramResult.isSuccess()) {
            return paramResult;
        }
        if (userService.isPhoneRepeat(null, userDTO.getPhoneNumber(), userDTO.getEnterpriseId(), null)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "手机号码不能重复");
        }
        if (userDTO.getPasswordType() == null) {
            //默认自定义创建密码类型
            userDTO.setPasswordType(2);
        }

        if (StringUtils.isBlank(userDTO.getPhoneAreaCode())) {
            userDTO.setPhoneAreaCode("86");
        }
        Integer userId = userService.saveUser(userDTO);
        return ServiceResponse.success(userId);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public ServiceResponse editUser(UserDTO userDTO) {
        //valid param
        ServiceResponse paramResult = ValidUtil.allCheckValidate(userDTO, UserDTO.EditUserValid.class);
        if (!paramResult.isSuccess()) {
            return paramResult;
        }
        TabSysUser tabUser = userService.getUserById(userDTO.getUserId());
        if (tabUser == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "用户ID输入有误");
        }
        //gic管理员只有一个超管
        if (userDTO.getSuperAdmin() != null && userDTO.getSuperAdmin().intValue() == 1
                && tabUser.getLoginType().intValue() == LoginUserTypeEnum.GIC_USER.getCode()) {
            //超级官员
            TabSysUser adminUser = userService.getUserByEnterpriseId(userDTO.getEnterpriseId());
            if (adminUser != null && adminUser.getUserId().intValue() != tabUser.getUserId().intValue()) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "已存在超级管理员，参数有误");
            }
        }

        ServiceResponse validParamRes = validParamOfSaveOrUpdate(userDTO);
        if (!validParamRes.isSuccess()) {
            return ServiceResponse.failure(validParamRes.getCode(), validParamRes.getMessage());
        }

        userService.editUser(userDTO);

        userRoleService.deleteByUserId(userDTO.getUserId());
        userResourceService.deleteByUserId(userDTO.getUserId());
        //如果账号分组有数据，则需要维护分组数据
        if (StringUtils.isNotBlank(userDTO.getAccountGroupIds())) {
            accountGroupRelService.deleteByUserId(userDTO.getUserId(), AccountGroupMemberTypeEnum.ADMIN.getCode());
        }

        //同步审核员
        auditorApiService.syncModifyNameOrPhone(userDTO.getUserId(), tabUser.getEnterpriseId(), userDTO.getPhoneNumber(), userDTO.getUserName(), null);

        ServiceResponse response = saveRole(userDTO.getUserId(), userDTO.getEnterpriseId(), userDTO.getUserRoleIds(),
                userDTO.getUserResourceIds(), userDTO.getAccountGroupIds());
        if (response.isSuccess()) {
            return response;
        } else {
            //事物回滚
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return response;
        }
    }

    private ServiceResponse validParamOfSaveOrUpdate(UserDTO userDTO) {

        if (userService.isPhoneRepeat(userDTO.getUserId(), userDTO.getPhoneNumber(), userDTO.getEnterpriseId(), null)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "手机号码不能重复");
        }

        Integer accountType = userDTO.getAccountType();
        boolean isNotRightAccountType = accountType != null && accountType != 1 && accountType != 2;

        if (isNotRightAccountType) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "管理员账号类型值非法");
        }
        boolean isTempAccount = accountType != null && accountType == 2;
        if (isTempAccount && StringUtils.isBlank(userDTO.getAccountDueDateStr())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "临时账号必须填写账号到期时间");
        }

        if (isTempAccount) {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            try {
                userDTO.setAccountDueDate(sdf.parse(userDTO.getAccountDueDateStr()));
            } catch (ParseException e) {
                userDTO.setAccountDueDate(new Date());
                LOGGER.warn("新建临时账号到期时间解析:{}", e.getMessage(), e);
            }
        }

        if (!AccountPositionEnum.isRightCode(userDTO.getUserPosition())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "职位值非法");
        }

        if (StringUtils.isBlank(userDTO.getPhoneAreaCode())) {
            userDTO.setPhoneAreaCode(Constants.NATION_CODE);
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> editAdmin(UserDTO userDTO) {
        //超级官员
        TabSysUser adminUser = userService.getUserByEnterpriseId(userDTO.getEnterpriseId());
        if (adminUser == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "超级管理员不存在，操作有误");
        }
        userDTO.setUserId(adminUser.getUserId());
        userService.editUser(userDTO);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<UserDTO> getUserById(Integer userId) {
        TabSysUser tabUser = userService.getUserById(userId);
        if (tabUser == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "用户ID输入有误");
        }
        if (tabUser.getLoginType().intValue() != LoginUserTypeEnum.GIC_USER.getCode()) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "用户类型错误，不是GIC管理员");
        }
        UserDTO result = EntityUtil.changeEntityNew(UserDTO.class, tabUser);
        //受审组
        List<Integer> userGroupIdList = auditedGroupUserRelService.listAuditedGroupByUserId(tabUser.getEnterpriseId(),
                userId);
        result.setUserGroupIds(Optional.ofNullable(userGroupIdList).orElse(new ArrayList<>()).stream()
                .map(e -> e.toString()).collect(Collectors.joining(",")));
        //角色集
        List<UserRoleDTO> userRoleDTOList = userRoleService.listUserRoleByUserId(userId);
        StringBuilder userRoleStr = new StringBuilder();
        for (UserRoleDTO userRoleDTO : userRoleDTOList) {
            userRoleStr.append(userRoleDTO.getRoleId()).append(",");
        }
        String userRoleResult = userRoleStr.toString();
        if (StringUtils.isBlank(userRoleResult)) {
            result.setUserRoleIds("");
        } else {
            result.setUserRoleIds(userRoleResult.substring(0, userRoleResult.length() - 1));
        }
        //资源集
        List<UserResourceDTO> userResourceDTOList = userResourceService.listUserResourceByUserId(userId);
        StringBuilder userResourceStr = new StringBuilder();
        for (UserResourceDTO userResourceDTO : userResourceDTOList) {
            userResourceStr.append(userResourceDTO.getResourceId()).append(",");
        }
        String userResourceResult = userResourceStr.toString();
        if (StringUtils.isBlank(userResourceResult)) {
            result.setUserResourceIds("");
        } else {
            result.setUserResourceIds(userResourceResult.substring(0, userResourceResult.length() - 1));
        }
        //账号分组
        List<TabSysAccountGroupRel> tabSysAccountGroupRelList = accountGroupRelService.listByUserId(userId,
                AccountGroupMemberTypeEnum.ADMIN.getCode());
        StringBuilder accountGroupIdStr = new StringBuilder();
        if (CollectionUtils.isNotEmpty(tabSysAccountGroupRelList)) {
            for (TabSysAccountGroupRel accountGroupRel : tabSysAccountGroupRelList) {
                accountGroupIdStr.append(accountGroupRel.getAccountGroupId()).append(",");
            }
        }
        String accountGroupIdResult = accountGroupIdStr.toString();
        if (StringUtils.isBlank(accountGroupIdResult)) {
            result.setAccountGroupIds("");
        } else {
            result.setAccountGroupIds(accountGroupIdResult.substring(0, accountGroupIdResult.length() - 1));
        }
        return ServiceResponse.success(result);
    }

    @Override
    public ServiceResponse<UserDTO> getUserInfoById(Integer userId) {
        TabSysUser tabUser = userService.getUserById(userId);
        if (tabUser == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "用户ID输入有误，无此数据");
        }
        return ServiceResponse.success(EntityUtil.changeEntityNew(UserDTO.class, tabUser));
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ServiceResponse<String> delete(Integer userId) {
        TabSysUser tabUser = userService.getUserById(userId);
        if (tabUser == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "用户ID输入有误，无此数据");
        }
        if (tabUser.getLoginType().intValue() != LoginUserTypeEnum.GIC_USER.getCode()) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(),
                    "用户类型错误，不是" + LoginUserTypeEnum.GIC_USER.getMessage());
        }

        //验证是否有创建的数据
        ServiceResponse<List<Integer>> userDataResponse = listManagerDataScene(userId);
        if (userDataResponse.isSuccess() && CollectionUtils.isNotEmpty(userDataResponse.getResult())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "用户有自己创建的数据，请先转移");
        }

        //删除角色关联
        userRoleService.deleteByUserId(userId);
        //删除资源关联
        userResourceService.deleteByUserId(userId);
        //删除账号分组关联
        accountGroupRelService.deleteByUserId(userId, AccountGroupMemberTypeEnum.ADMIN.getCode());
        //受审组
        auditedGroupUserRelService.deleteByUserId(userId);
        userService.delete(userId);
        return ServiceResponse.success(tabUser.getUserName());
    }

    @Override
    public ServiceResponse<UserDTO> getUserByEnterpriseId(Integer enterpriseId) {
        TabSysUser tabUser = userService.getUserByEnterpriseId(enterpriseId);
        if (tabUser == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID输入有误");
        }
        return ServiceResponse.success(EntityUtil.changeEntityNew(UserDTO.class, tabUser));
    }

    @Override
    public ServiceResponse<List<UserDTO>> listUserByPhoneNumber(String phoneNumber, String nationCode) {
        List<TabSysUser> list = this.userService.listAllUserByPhoneNumber(phoneNumber, nationCode);
        if (CollectionUtils.isNotEmpty(list)) {
            return ServiceResponse.success(EntityUtil.changeEntityListByJSON(UserDTO.class, list));
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<UserDTO> listUserByPhoneNumber(String phoneNumber, Integer enterpriseId) {
        if (enterpriseId == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户数据为空");
        }
        List<TabSysUser> list = this.userService.listAllUserByPhoneNumber(phoneNumber, Constants.NATION_CODE);
        if (CollectionUtils.isNotEmpty(list)) {
            for (TabSysUser user : list) {
                if (enterpriseId.intValue() == user.getEnterpriseId().intValue()) {
                    return ServiceResponse.success(EntityUtil.changeEntityNew(UserDTO.class, user));
                }
            }
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<UserDTO> listUserByPhoneNumber(String nationCode, String phoneNumber, Integer enterpriseId) {
        if (enterpriseId == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户数据为空");
        }
        List<TabSysUser> list = this.userService.listUserByPhoneNumberAndEnterprise(phoneNumber, nationCode,
                enterpriseId);
        if (CollectionUtils.isNotEmpty(list)) {
            return ServiceResponse.success(EntityUtil.changeEntityNew(UserDTO.class, list.get(0)));
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<UserDTO> getUserByPhoneNumber(String nationCode, String phoneNumber, Integer enterpriseId) {
        if (enterpriseId == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户数据为空");
        }
        List<TabSysUser> list = this.userService.listUserByPhoneNumberAndEnterprise(phoneNumber, nationCode,
                enterpriseId);
        if (CollectionUtils.isNotEmpty(list)) {
            for (TabSysUser tabSysUser : list) {
                if (tabSysUser.getLoginType() == 1) {
                    return ServiceResponse.success(EntityUtil.changeEntityNew(UserDTO.class, tabSysUser));
                }
            }
        }
        return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "查不到用户数据");
    }

    @Override
    public ServiceResponse<UserDTO> getGicUserByPhoneNumber(String nationCode, String phoneNumber, Integer enterpriseId) {
        if (enterpriseId == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户数据为空");
        }
        List<TabSysUser> list = this.userService.listUserByPhoneNumberAndEnterprise(phoneNumber, nationCode,
                enterpriseId);
        if (CollectionUtils.isNotEmpty(list)) {
            for (TabSysUser tabSysUser : list) {
                //gic用户
                if (tabSysUser.getLoginType() == 0) {
                    return ServiceResponse.success(EntityUtil.changeEntityNew(UserDTO.class, tabSysUser));
                }
            }
        }
        return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "查不到用户数据");
    }

    @Override
    public ServiceResponse<UserDTO> login(String nationCode, String phoneNumber, Integer enterpriseId,
            String password) {
        TabSysUser user = this.userService.login(nationCode, phoneNumber, enterpriseId, password);
        return ServiceResponse.success(EntityUtil.changeEntityByJSON(UserDTO.class, user));
    }

    @Override
    public ServiceResponse<Page<UserListDTO>> pageUser(UserListQO params) {
        if (params.getAccountGroupId() != null) {
            //查询分组下的管理员列表
            List<TabSysAccountGroupRel> list = accountGroupRelService.listByAccountGroupId(params.getAccountGroupId(),
                    AccountGroupMemberTypeEnum.ADMIN.getCode());
            if (CollectionUtils.isNotEmpty(list)) {
                List<Integer> userIdList = new ArrayList<>(list.size());
                for (TabSysAccountGroupRel temp : list) {
                    userIdList.add(temp.getUserId());
                }
                params.setUserIdList(userIdList);
            }
            //如果是空的，则返回空
            if (CollectionUtils.isEmpty(params.getUserIdList())) {
                return ServiceResponse.success(new Page<>());
            }

        }
        com.github.pagehelper.Page page = userService.pageUser(params);
        Page<UserListDTO> resultPage = PageHelperUtils.changePageHelperToCurrentPage(page);
        return ServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse<List<UserListDTO>> listUser(UserListQO params) {
        List<UserListDTO> list = userService.listUser(params);
        return ServiceResponse.success(list);
    }

    @Override
    public ServiceResponse<List<UserDTO>> listUserByIdList(List<Integer> userIdList) {
        List<TabSysUser> list = userService.listUserByIdList(userIdList);
        if (CollectionUtils.isNotEmpty(list)) {
            return ServiceResponse.success(EntityUtil.changeEntityListNew(UserDTO.class, list));
        }
        return ServiceResponse.success(new ArrayList<>());
    }

    @Override
    public ServiceResponse<Object> getUserDetail(String token) {
        String key = Constants.TOKEN_KEY + token;
        Object cache = RedisUtil.getCache(key);
        if (cache == null) {
            return ServiceResponse.success();
        }
        return ServiceResponse.success(cache);
    }

    @Override
    public ServiceResponse<String> removeAccountGroup(Integer accountGroupId, Integer userId, Integer type) {
        TabSysUser tabUser = userService.getUserById(userId);
        if (tabUser == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "用户ID输入有误，无此数据");
        }
        if (tabUser.getLoginType().intValue() != LoginUserTypeEnum.GIC_USER.getCode()) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(),
                    "用户类型错误，不是" + LoginUserTypeEnum.GIC_USER.getMessage());
        }
        TabSysAccountGroup tabSysAccountGroup = accountGroupService.getById(accountGroupId);
        if (tabSysAccountGroup == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "账号分组ID输入有误，无此数据");
        }
        accountGroupRelService.deleteByAccountGroupIdAndUserId(userId, accountGroupId, type);
        return ServiceResponse.success(tabUser.getUserName());
    }

    @Override
    public ServiceResponse<String> editPhoneOrPassword(Integer userId, String nationCode, String phone,
            String password) {
        TabSysUser tabUser = userService.getUserById(userId);
        if (tabUser == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "用户ID输入有误，无此数据");
        }
        UserDTO userDTO = new UserDTO();
        userDTO.setUserId(userId);
        userDTO.setPhoneNumber(phone);
        userDTO.setPhoneAreaCode(nationCode);
        userDTO.setPassword(password);
        userService.editUser(userDTO);

        //同步审核员
        auditorApiService.syncModifyNameOrPhone(userId, tabUser.getEnterpriseId(), phone, null, null);
        return ServiceResponse.success(tabUser.getUserName());
    }

    @Override
    public ServiceResponse<List<Integer>> listAllUser(Integer enterpriseId) {
        return ServiceResponse.success(userService.listAllUser(enterpriseId));
    }

    @Override
    public ServiceResponse<List<UserDTO>> listGicUser(Integer enterpriseId) {
        List<TabSysUser> list = userService.listGicUser(enterpriseId, 0);
        if (CollectionUtils.isNotEmpty(list)) {
            return ServiceResponse.success(EntityUtil.changeEntityListNew(UserDTO.class, list));
        }
        return ServiceResponse.success(Collections.emptyList());
    }

    @Override
    public ServiceResponse<List<UserDTO>> listOperationUser(Integer enterpriseId) {
        List<TabSysUser> list = userService.listGicUser(enterpriseId, 1);
        if (CollectionUtils.isNotEmpty(list)) {
            return ServiceResponse.success(EntityUtil.changeEntityListNew(UserDTO.class, list));
        }
        return ServiceResponse.success(Collections.emptyList());
    }

    @Override
    public ServiceResponse<List<UserDTO>> listUser(Integer enterpriseId) {
        List<TabSysUser> list = userService.listGicUser(enterpriseId, null);
        if (CollectionUtils.isNotEmpty(list)) {
            return ServiceResponse.success(EntityUtil.changeEntityListNew(UserDTO.class, list));
        }
        return ServiceResponse.success(Collections.emptyList());
    }

    @Override
    public ServiceResponse<List<Integer>> listManagerDataScene(Integer userId) {
        List<Integer> result = new ArrayList<>();
        Integer pageNum = 1;
        Integer pageSize = 1;

        Page<UserDataShowDTO> page;
        for (ManagerDataEnum managerDataEnum : ManagerDataEnum.values()) {
            page = ManagerDataUtils.queryDataFromLocal(managerDataEnum.getCode(), userId, pageNum, pageSize);
            boolean hasData = page != null && CollectionUtils.isNotEmpty(page.getResult());
            if (hasData) {
                result.add(managerDataEnum.getCode());
            }
        }
        return ServiceResponse.success(result);
    }

    @Override
    public ServiceResponse<Page<UserDataShowDTO>> queryUserManagerData(Integer dataScene, Integer userId,
                                                                          Integer pageNum, Integer pageSize) {
        return ServiceResponse.success(ManagerDataUtils.queryDataFromLocal(dataScene, userId, pageNum, pageSize));
    }

    @Override
    public ServiceResponse<Page<UserDTO>> queryUserTransferObject(Integer userId, String search,
                                                                  Integer pageNum, Integer pageSize) {
        TabSysUser tabUser = userService.getUserById(userId);
        if (tabUser == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "用户ID输入有误，无此数据");
        }
        List<Integer> transferUserList = queryCouldTransferUserList(userId, tabUser.getEnterpriseId());

        UserListQO params = new UserListQO();
        params.setUserIdList(transferUserList);
        params.setSearch(search);
        params.setCurrentPage(pageNum);
        params.setPageSize(pageSize);

        com.github.pagehelper.Page page = userService.pageUserForTransfer(params);
        Page<UserDTO> resultPage = PageHelperUtils.changePageHelperToCurrentPage(page, UserDTO.class);

        return ServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse<Void> transferUserData(Integer oldUserId, Integer newUserId) {
        TabSysUser tabUser = userService.getUserById(oldUserId);
        if (tabUser == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "用户ID输入有误，无此数据");
        }
        List<Integer> transferUserList = queryCouldTransferUserList(oldUserId, tabUser.getEnterpriseId());
        if (!transferUserList.contains(newUserId)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "转移对象不符合条件");
        }
        //转移
        userMoveApiService.saveUserMove(tabUser.getEnterpriseId(), oldUserId, newUserId);
        //发布消息，业务方自己调用
        transferUserDataMQ(oldUserId, newUserId);
        return ServiceResponse.success();
    }

    private void transferUserDataMQ(Integer oldUserId, Integer newUserId) {
        GicMQClient client = GICMQClientUtil.getClientInstance();
        Map<String, Object> params = new HashMap<>(4);
        params.put("oldUserId", oldUserId);
        params.put("newUserId", newUserId);
        try {
            client.sendMessage("transferUserDataMQ", JSONObject.toJSONString(params));
        } catch (Exception e) {
            LOGGER.warn("管理员转移数据路由:{}", e.getMessage(), e);
        }
    }

    /**
     * 查询符合条件的转移对象
     * @param userId
     * @param enterpriseId
     * @return
     */
    private List<Integer> queryCouldTransferUserList(Integer userId, Integer enterpriseId) {
        TabSysUser admin = userService.getUserByEnterpriseId(enterpriseId);
        //前置条件 权限集和资源组大于等于的管理员（包括超管）
        //角色集
        List<UserRoleDTO> userRoleDTOList = userRoleService.listUserRoleByUserId(userId);
        List<Integer> roleIdList = new ArrayList<>();
        for (UserRoleDTO userRoleDTO : userRoleDTOList) {
            roleIdList.add(userRoleDTO.getRoleId());
        }
        //权限集大于等于的管理员
        List<Integer> userIdListByRole = userRoleService.listUserId(roleIdList, enterpriseId);

        List<Integer> resourceIdList = new ArrayList<>();
        List<UserResourceDTO> userResourceDTOList = userResourceService.listUserResourceByUserId(userId);
        for (UserResourceDTO userResourceDTO : userResourceDTOList) {
            resourceIdList.add(userResourceDTO.getResourceId());
        }
        //资源组大于等于的管理员
        List<Integer> userIdListByResource = userResourceService.listUserId(resourceIdList, enterpriseId);

        //取交集，因为是并且关系
        userIdListByRole.retainAll(userIdListByResource);

        //超管ID
        userIdListByRole.add(admin.getUserId());
        //删除转移的用户
        userIdListByRole.remove(userId);
        return userIdListByRole;
    }

    /**
     * 保存关联数据，角色关联、资源关联
     * @Title: saveRole 
     * @Description:
      * @author guojuxing
     * @param userId
     * @param enterpriseId
     * @param userRoleIds
     * @param userResourceIds 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Integer> 
      */
    private ServiceResponse<Integer> saveRole(Integer userId, Integer enterpriseId, String userRoleIds,
            String userResourceIds, String accountGroupIds) {
        String[] userRoleIdArr;
        String[] userResourceIdArr;
        String[] accountGroupIdArr;
        ServiceResponse<String[]> userRoleValid = ValidSplitUtils.validStr(userRoleIds);
        if (userRoleValid.isSuccess()) {
            userRoleIdArr = userRoleValid.getResult();
        } else {
            return ServiceResponse.failure(userRoleValid.getCode(), userRoleValid.getMessage());
        }
        ServiceResponse<String[]> userResourceIdValid = ValidSplitUtils.validStr(userResourceIds);
        if (userResourceIdValid.isSuccess()) {
            userResourceIdArr = userResourceIdValid.getResult();
        } else {
            return ServiceResponse.failure(userResourceIdValid.getCode(), userResourceIdValid.getMessage());
        }
        //新增用户角色关联数据
        UserRoleDTO userRoleDTO = new UserRoleDTO();
        userRoleDTO.setEnterpriseId(enterpriseId);
        userRoleDTO.setUserId(userId);
        for (String userRole : userRoleIdArr) {
            int roleId = Integer.parseInt(userRole);
            TabSysRole tabSysRole = roleService.getById(roleId);
            if (tabSysRole == null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "角色数据有误，无此角色数据");
            }
            userRoleDTO.setRoleId(roleId);
            ServiceResponse result = userRoleApiService.save(userRoleDTO);
            if (!result.isSuccess()) {
                return result;
            }
        }
        //新增用户资源关联数据
        UserResourceDTO userResourceDTO = new UserResourceDTO();
        userResourceDTO.setUserId(userId);
        userResourceDTO.setEnterpriseId(enterpriseId);
        for (String userResource : userResourceIdArr) {
            int resourceId = Integer.parseInt(userResource);
            TabSysResourceGroup tabSysResource = resourceGroupService.getResourceGroup(resourceId);
            if (tabSysResource == null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "资源数据有误，无此资源数据");
            }
            userResourceDTO.setResourceId(resourceId);
            ServiceResponse result = userResourceApiService.save(userResourceDTO);
            if (!result.isSuccess()) {
                return result;
            }
        }
        //账号分组用户关联数据维护
        if (StringUtils.isNotBlank(accountGroupIds)) {
            ServiceResponse<String[]> accountGroupIdValid = ValidSplitUtils.validStr(accountGroupIds);
            if (accountGroupIdValid.isSuccess()) {
                accountGroupIdArr = accountGroupIdValid.getResult();
            } else {
                return ServiceResponse.failure(accountGroupIdValid.getCode(), accountGroupIdValid.getMessage());
            }
            List<AccountGroupRelDTO> accountGroupRelDTOList = new ArrayList<>();
            Date now = new Date();
            for (String accountGroup : accountGroupIdArr) {
                int accountGroupId = Integer.parseInt(accountGroup);
                TabSysAccountGroup tabSysAccountGroup = accountGroupService.getById(accountGroupId);
                if (tabSysAccountGroup == null) {
                    return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "账号分组有误，无此数据");
                }
                AccountGroupRelDTO accountGroupRelDTO = new AccountGroupRelDTO();
                accountGroupRelDTO.setType(AccountGroupMemberTypeEnum.ADMIN.getCode());
                accountGroupRelDTO.setUserId(userId);
                accountGroupRelDTO.setAccountGroupId(accountGroupId);
                accountGroupRelDTO.setCreateTime(now);
                accountGroupRelDTO.setUpdateTime(now);
                accountGroupRelDTO.setEnterpriseId(enterpriseId);
                accountGroupRelDTO.setDeleteFlag(DeleteFlagConstants.NORMAL_STATUS);
                accountGroupRelDTOList.add(accountGroupRelDTO);
            }
            accountGroupRelService.insertFetch(accountGroupRelDTOList);
        }
        return ServiceResponse.success(userId);
    }

}
